# Main BRT models for single species using R-package 'dismo'
# 
# Author: Johannes Radinger
###############################################################################

##### LOAD BASICS (libs, dirs) ######
# Load libraries
library(dismo)
library(raster)

#Set Data directories
maindir <- "/path/to/data/Treene/SDM_Treene/SDM_BRT_Treene"
fish_dir <- "/path/to/data/Treene/Fish_Data/"
results_dir <- paste(maindir,"Results/",sep="")
predictors_dir <- paste(maindir,"Predictors/",sep="")



####### LOAD PREDICTORS AND SPECIES #########
# Define Predictorset
set <- "Set_A_point"

# Load Predictors raster stack
predictors <- readRDS(file = paste(predictors_dir,set,".rds",sep=""))
#Load treene.raster and calculate mask
treene.raster <- raster(paste(predictors_dir,"Treene_raster.tif",sep=""))
fun.mask <- function(x) {x[!is.na(x)] <- 1; return(x)}
treene.mask <- calc(sum(predictors,treene.raster), fun.mask) ## get 1 where all maps !NA

# Load species data as spatial points dataframe
dfm.fish.unmask <- readRDS(file = paste(fish_dir,"dfm.fish.unmask.rds",sep=""))

#Select only points within MASK (only points where all predictors are available)
dfm.fish.maskindex <- extract(treene.mask,dfm.fish.unmask)
dfm.fish <- dfm.fish.unmask[!is.na(dfm.fish.maskindex),]

# only select species with more than x(=10) observations
select.species <- table(as.data.frame(dfm.fish)[dfm.fish$value>0,"Spec_Code"])[table(as.data.frame(dfm.fish)[dfm.fish$value>0,"Spec_Code"])>=10]
select.species.list <- as.list(names(select.species))

#Write mask
#writeRaster(treene.mask,paste(predictors_dir,"Treene_raster_mask.tif",sep=""),overwrite=TRUE)

########## SDM runs for single species #############

# set species
species.list <- select.species.list[c(2,4,8,9,10)]
names(predictors)
# Set number of replicates - kfold-cross validation
k <- 10


################### Boosted regression trees ###########################
# function for removing columns from a data.frame with no variation
rm.col <- function(df){
  apply(df, 2, function(x) length(unique(x)) > 1)
}

for(i in species.list){		
	message(paste("Model for species: ",i,sep=""))
	#Create directory for results
	subdir <- paste(set,"/",i,"/BRT",sep="")
	dir.create(paste(results_dir,subdir,sep=""),recursive=TRUE)
	
	# Preparing  Dataset
	dfm.fish.brt <- dfm.fish[dfm.fish$Spec_Code ==  i,c("Spec_Code","value")]
	# reclass value into pres/abs (0/1)
	dfm.fish.brt$value <- 1*(dfm.fish.brt$value>0)
	#extracting environmental variables
	brt.vals <- extract(predictors,dfm.fish.brt)
	#combining value with predictor dataframe
	brt.data <- data.frame(cbind(value=dfm.fish.brt$value,brt.vals))
	#define categorial variables as factors
	ix <- grep("^crast", colnames(brt.data))
	brt.data[, ix] <- lapply(brt.data[, ix], as.factor)

	#update data, selecting only those columns with variation
	brt.data <- brt.data[,rm.col(brt.data)]
	summary(brt.data)
	
	# Global BRT Model
	message("Step 1: Global Model")
	brt.model.global <-  gbm.step(data = brt.data,gbm.x = 2:length(brt.data), gbm.y = 1,
			family = "bernoulli", tree.complexity = 3, learning.rate = 0.001)
	saveRDS(brt.model.global,file=paste(results_dir,subdir,"/brt.model.global",sep=""))
	
	# Model simplification based on gbm.simplify 
	message("Step 2: Model Simplification")
	model.simplify <- gbm.simplify(brt.model.global) #n.drops=length(brt.data[,-1])-(5) #(remove up to 5 remaining variables)
	drops <- which.min(model.simplify$deviance.summary$mean)
	
	# Variable selection
	simple.predictors <- model.simplify$pred.list[[drops]] # index numbers
	simple.predictor.names <- model.simplify$gbm.call$predictor.names[simple.predictors-1]
	

	# Model Prediction / Map
	message("Step 3: Final Model")
	brt.model.final <-  gbm.step(data = brt.data,gbm.x = simple.predictors, gbm.y = 1, n.folds = k,
		family = "bernoulli", tree.complexity = 3, learning.rate = 0.001)
	saveRDS(brt.model.final,file=paste(results_dir,subdir,"/brt.model.final",sep=""))
	brt.predict <- predict(object = predictors[[simple.predictor.names]], model = brt.model.final, progress='text', 
			n.trees=model.simplify$gbm.call$best.trees, type="response", filename= paste(results_dir,subdir,"/prediction",sep=""), format="GTiff",
			overwrite=TRUE)
	
	#Species response plots
	pdf(paste(results_dir,subdir,"/species_response.pdf",sep=""))
	par(mfrow=c(3,4))
	gbm.plot(brt.model.final,n.plots=12,write.title=F)
	dev.off()
	
	# Create summary statistics
	message("Step 4: Final Model Statistics")
	# Variable contribution
	VC <- summary(brt.model.final)
	
	ROC.train <- brt.model.final$self.statistics$discrimination
	ROC.validation <- brt.model.final$cv.statistics$discrimination.mean
	ROC.validation.se <- brt.model.final$cv.statistics$discrimination.se
	ROC.stats.final <- cbind(ROC.train,ROC.validation,ROC.validation.se)
	
	brt.stats.final <- list(species=i,Date=Sys.time(),kfold=k,Variable.contribution=VC,ROC.statistics=ROC.stats.final)
	saveRDS(brt.stats.final, file = paste(results_dir,subdir,"/brt.stats.final",sep=""))
}






