# Analysis of the final model's AUC. Comparisons between modelled distance classes, topological variables
# and measured vs. assessed hydromorphological variables
# 
# Author: Johannes Radinger
###############################################################################

##### LOAD BASICS (libs, dirs) ######
# Load libraries
library(reshape2)
library(lme4)
library(plyr)
library(xlsx)


#Resultsdir
maindir <- "/path/to/data/Treene/SDM_Treene/SDM_BRT_Treene/"
results_dir <- paste(maindir,"Results/",sep="")
fish_dir <- "/path/to/data/Treene/Fish_Data/"

# Load AUC.df
AUC.df <- readRDS(paste(results_dir,"AUC_df.rds",sep=""))
head(AUC.df)


# Import Fishnames
species_info <-read.csv(paste(fish_dir,"Species_Info.csv",sep=""),header=T)


####### Summary stats ##########
# n = 260
nrow(AUC.df)

# mean and sd AUC
mean(AUC.df$AUC.validation,na.rm=TRUE)
sd(AUC.df$AUC.validation,na.rm=TRUE)

# model Qualities: >0.8 = good, >0.9 excellent
nrow(subset(AUC.df, AUC.validation>0.5))
nrow(subset(AUC.df, AUC.validation>=0.9))/nrow(AUC.df)

#summary stats for selected species
head(AUC.df)
AUC.df.selected.species <- AUC.df[AUC.df$species %in% c("Cobienia","Gastatus","Gobiobio","Leuciscus","Percilis","Phoxinus","Pungtius","Rutiilus"),]
summary(AUC.df.selected.species$AUC.validation)
min(AUC.df.selected.species$AUC.validation,na.rm=TRUE)
max(AUC.df.selected.species$AUC.validation,na.rm=TRUE)
sd(AUC.df.selected.species$AUC.validation,na.rm=TRUE)


############################ comparsions of means ################
## melt df
AUC.df.melt <- melt(AUC.df,measure.vars="AUC.validation",id.vars=c("distance","set.habitat","set.topology","species"))

# AUC summary df for MS
AUC_df_MS <- dcast(AUC.df.melt[AUC.df.melt$variable=="AUC.validation",], 
                   set.topology+species~distance+set.habitat)
write.xlsx(AUC_df_MS,paste(results_dir,"AUC_df_MS.xlsx",sep=""))


##################### set.topology #########################
# mean for +
mean(AUC.df$AUC.validation[AUC.df$set.topology=="+"],na.rm=TRUE)
sd(AUC.df$AUC.validation[AUC.df$set.topology=="+"],na.rm=TRUE)
quantile(AUC.df$AUC.validation[AUC.df$set.topology=="+"],na.rm=TRUE,probs=c(0.25,0.75))
# mean for no
mean(AUC.df$AUC.validation[AUC.df$set.topology=="no"],na.rm=TRUE)
sd(AUC.df$AUC.validation[AUC.df$set.topology=="no"],na.rm=TRUE)
quantile(AUC.df$AUC.validation[AUC.df$set.topology=="no"],na.rm=TRUE,probs=c(0.25,0.75))

#paired t-test: Alternative hypotheses: 
#The AUC is greater when topology is included than without ("+" > "no")
df.topology <- dcast(AUC.df.melt,species+distance+set.habitat~set.topology)
head(df.topology)

#Test for normality
shapiro.test(asin(sqrt(df.topology[,"+"]))) 
shapiro.test(asin(sqrt(df.topology[,"no"])))
# --> normal distribution after transformation


t.test(asin(sqrt(df.topology[,"+"])),
       asin(sqrt(df.topology[,"no"])),
       alternative="two.sided",
       paired=TRUE) #p=0.01059

df.topology$diff <- df.topology[,"+"]-df.topology[,"no"]

df.topology.length <- merge(df.topology,species_info,by.x="species",by.y="Spec_Code")
plot(df.topology.length$L,df.topology.length$diff)

cor.test(df.topology.length$L,df.topology.length$diff,method="spearman") # p =0.0065

##################### set.habitat #########################
# mean for MV
mean(AUC.df$AUC.validation[AUC.df$set.habitat=="MV"],na.rm=TRUE)
sd(AUC.df$AUC.validation[AUC.df$set.habitat=="MV"],na.rm=TRUE)
quantile(AUC.df$AUC.validation[AUC.df$set.habitat=="MV"],na.rm=TRUE,probs=c(0.25,0.75))
# mean for AV
mean(AUC.df$AUC.validation[AUC.df$set.habitat=="AV"],na.rm=TRUE)
sd(AUC.df$AUC.validation[AUC.df$set.habitat=="AV"],na.rm=TRUE)
quantile(AUC.df$AUC.validation[AUC.df$set.habitat=="AV"],na.rm=TRUE,probs=c(0.25,0.75))


#paired wilcox test: Alternative hypotheses: 
#The AUC is greater for Measured Variables than for Assessement values (A>FE)
df.habitat <- dcast(AUC.df.melt,species+distance+set.topology~set.habitat)

#Test for normality
shapiro.test(asin(sqrt(df.habitat[,"MV"]))) 
shapiro.test(asin(sqrt(df.habitat[,"AV"])))
# --> Normal distribution after transformation

# Test of homogeneity of variances
bartlett.test(asin(sqrt(value)) ~ distance, data=AUC.df.melt)

t.test(asin(sqrt(df.habitat[,"MV"])),
       asin(sqrt(df.habitat[,"AV"])),
       alternative="two.sided",
       var.equal=FALSE,
       paired=TRUE) # p=0.917

var.test(asin(sqrt(df.habitat[,"MV"])),
              asin(sqrt(df.habitat[,"AV"])),
              alternative="greater",
              )



# comparison between species
for(i in levels(droplevels(AUC.df$species))){
  print(i)
  
  t_res <- t.test(asin(sqrt(AUC.df$AUC.validation[AUC.df$set.habitat=="AV" & AUC.df$species==i])),
                  asin(sqrt(AUC.df$AUC.validation[AUC.df$set.habitat=="MV" & AUC.df$species==i])),
                  alternative="two.sided",
                  paired=TRUE) #p=0.01059
  print(t_res)
}

for(i in levels(droplevels(AUC.df$species))){
  print(i)
  
  wilcox_res <- wilcox.test(AUC.df$AUC.validation[AUC.df$set.habitat=="AV" & AUC.df$species==i],
                  AUC.df$AUC.validation[AUC.df$set.habitat=="MV" & AUC.df$species==i],
                  alternative="two.sided",
                  paired=TRUE) #p=0.01059
  print(wilcox_res)
}

##################### distance #########################
#paired wilcox test: Alternative hypotheses: 
#The AUC increases with distance
df.distance <- dcast(AUC.df.melt,species+set.topology+set.habitat~distance)



########################### mixed model ########################
# linear mixed model with species as random term
df_lmer <- AUC.df[!is.na(AUC.df$AUC.validation),]
df_lmer$AUC.validation.asin <- asin(sqrt(df_lmer$AUC.validation))

boxplot(df_lmer$AUC.validation.asin)
shapiro.test(df_lmer$AUC.validation.asin) 

mod_aov <- aov(AUC.validation.asin ~ (distance+ set.topology + set.habitat)**2 + Error(species), data =df_lmer[!(df_lmer$species %in% c("Gastatus","Tincinca")),])
summary(mod_aov)

mmod1<- lmer(AUC.validation.asin ~ (distance + set.topology + set.habitat )+ (1|species), data =df_lmer[!(df_lmer$species %in% c("Gastatus","Tincinca")),])
ci_mmod1 <- confint.merMod(mmod1,method="boot",nsim=1000)
summary(mmod1)



